#include "ratingMigrate.h"
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QVariant>
#include <QDebug>
#include <QDir>

RatingMigrate::RatingMigrate(QString prefix) {
    m_prefix = prefix;
}

QString RatingMigrate::url2path (QString url) const {
    if (url.startsWith(".")) url=url.mid(1);
    if (!url.startsWith(m_prefix)) url = m_prefix + url;
    return url;
}
//start id="select"
void RatingMigrate::readV1Ratings(QSqlDatabase db) {
    QString selstr = "select url,rating,createdate,accessdate from statistics where rating>0";
    QSqlQuery q = db.exec(selstr);
    if (!q.isActive() || !q.isSelect()) {
        qDebug() << "query failed. " << q.lastError().text();
        return;
    }
    while (q.next()) {
        QString url = q.value(0).toString();
        int rating = q.value(1).toInt();
        int accessDate = q.value(3).toInt();
//end
        StatsRec rec;
        QString path = url2path(url);
        QFileInfo fi = QFileInfo(path);
        if (!fi.exists()) {
            qDebug() << " not found: " << path<< rating;
            continue;
        }
        if (m_map.contains(fi.absoluteFilePath())) {
            rec = m_map[fi.absoluteFilePath()];
            if (rec.rating != rating) {
                if (rec.accessDate > accessDate) {
                    qDebug() << "Rating different but older accessdate. " << url
                            << " current: " << rec.rating << " ignored: " <<  rating;
                    continue;
                }
            }
        }
        rec.fi = fi;
        rec.rating = rating;
        rec.createDate = q.value(2).toInt();
        rec.accessDate = accessDate;
//        qDebug() << rec.fi.absoluteFilePath() << '\t' << rec.rating;
        m_map[rec.fi.absoluteFilePath()] = rec;
    }
    qDebug() << " Found " << m_map.size() << " records";
}

void RatingMigrate::readV2Ids(QSqlDatabase db) {

    QString queryStr = "select id, rpath from urls";
    QSqlQuery q = db.exec(queryStr);
    if (!q.isActive() || !q.isSelect()) {
        qDebug() << "query failed. " << q.lastError().text();
        return;
    }
    while (q.next()) {
        int id = q.value(0).toInt();
        QString url = q.value(1).toString();
        url = url2path(url);
        QFileInfo fi(url);
        QString path = fi.absoluteFilePath();
        if (!fi.exists()) {
            qDebug() << "Not Found: " << path;
            continue;
        }
        if (!m_map.contains(path)) {
            qDebug() << "Ignoring: " << path;
            continue;
        }
        m_map[path].id = id;
    }
}
//start id="insert"
void RatingMigrate::writeV2Ratings(QSqlDatabase db) {
    QString queryStr = "insert INTO statistics (url, rating, createdate, accessdate) "
            " VALUES(%1, %2, %3, %4) ON DUPLICATE KEY UPDATE "
            " rating=VALUES(rating) ";
    int count =0;
    foreach (StatsRec rec, m_map.values()) {
        if (rec.id == -1) {
            qDebug() << "ERR! No id for " << rec.fi.absoluteFilePath();
            continue;
        }
        QSqlQuery q = db.exec(queryStr.arg(rec.id).arg(rec.rating).arg(rec.createDate).arg(rec.accessDate));
        if (!q.isActive()) {
            qDebug() << "failed query: " << q.lastQuery() << q.lastError().text();
        }
//end
        count ++;
    }
    qDebug() << "executed " << count << "inserts.";
    db.commit();
}
